/**
 * Vanilla JS FocusPoint; version: 1.2
 * Original Script Author: http://jonathonmenz.com
 * Source: https://github.com/jonom/jquery-focuspoint
 *
 * Updated to Vanilla JS by ChatGPT-4o.
 */
(function (Drupal, once, document, window) {
  'use strict';

  Drupal.behaviors.focusPoint = {
    attach: function (context, drupalSettings) {
      const focuspoints = once('focuspointfocusbehavior', '.focuspoint', context);
      focuspoints.forEach(function (focuspoint) {
        focusPoint(focuspoint, {
          throttleDuration: 20
        }).adjustFocus();
      });

      let hasFocalpointBreakpoints = drupalSettings.hasOwnProperty("focalpoint-breakpoints");
      if (hasFocalpointBreakpoints) {
        window.addEventListener("orientationchange", focuspointBreakpointsRespond);
        window.addEventListener("resize", focuspointBreakpointsRespond);
        window.addEventListener("load", focuspointBreakpointsRespond);
        document.addEventListener("drupal.display", focuspointBreakpointsRespond);
      }
    }
  };

  let focuspointBreakpointsRespond = function () {
    let focuspointBreakpoints = drupalSettings['focalpoint-breakpoints'];

    for (let itemNamespace in focuspointBreakpoints) {
      let breakpoints = focuspointBreakpoints[itemNamespace];
      let focuspoints = document.querySelectorAll('.focuspoint-' + itemNamespace);

      focuspoints.forEach(function (focuspoint) {
        for (let breakpoint in breakpoints) {
          let { mediaQuery, height, label } = breakpoints[breakpoint];
          if (window.matchMedia(mediaQuery).matches) {
            focuspoint.style.height = height + 'px';
            focuspoint.setAttribute('data-active-breakpoint', label);
            focuspoint.setAttribute('data-height', height);
            focusPoint(focuspoint).adjustFocus();
          }
        }
      });
    }
  };

  function focusPoint(el, settings = {}) {
    settings = Object.assign({ throttleDuration: 17 }, settings);

    var thrAdjustFocus = settings.throttleDuration
      ? throttle(function () { adjustFocus(el); }, settings.throttleDuration)
      : function () { adjustFocus(el); };
    var isListening = false;

    adjustFocus(el);

    return {
      adjustFocus: function () {
        return adjustFocus(el);
      },
      windowOn: function () {
        if (isListening) return;
        window.addEventListener('resize', thrAdjustFocus);
        isListening = true;
      },
      windowOff: function () {
        if (!isListening) return;
        window.removeEventListener('resize', thrAdjustFocus);
        isListening = false;
      }
    };
  }

  function adjustFocus(el) {
    var imageW = el.dataset.imageW;
    var imageH = el.dataset.imageH;
    var imageSrc = el.dataset.imageSrc;

    if (!imageW && !imageH && !imageSrc) {
      return setupContainer(el);
    }

    var containerW = el.offsetWidth;
    var containerH = el.offsetHeight;
    var focusX = parseFloat(el.dataset.focusX);
    var focusY = parseFloat(el.dataset.focusY);
    var img = el.querySelector('img');

    var hShift = 0;
    var vShift = 0;

    if (!(containerW > 0 && containerH > 0 && imageW > 0 && imageH > 0)) {
      return false;
    }

    var wR = imageW / containerW;
    var hR = imageH / containerH;

    img.style.maxWidth = '';
    img.style.maxHeight = '';

    if (imageW > containerW && imageH > containerH) {
      img.style[wR > hR ? 'maxHeight' : 'maxWidth'] = '100%';
    }

    if (wR > hR) {
      hShift = calcShift(hR, containerW, imageW, focusX);
    } else if (wR < hR) {
      vShift = calcShift(wR, containerH, imageH, focusY, true);
    }

    img.style.top = vShift;
    img.style.left = hShift;
  }

  function setupContainer(el) {
    var img = el.querySelector('img');
    var imageSrc = img.src;
    el.dataset.imageSrc = imageSrc;

    resolveImageSize(imageSrc, function (err, dim) {
      el.dataset.imageW = dim.width;
      el.dataset.imageH = dim.height;
      adjustFocus(el);
    });
  }

  function resolveImageSize(src, cb) {
    var img = new Image();
    img.onload = function () {
      cb(null, {
        width: img.width,
        height: img.height
      });
    };
    img.src = src;
  }

  function throttle(fn, ms) {
    var isRunning = false;
    return function () {
      var args = Array.prototype.slice.call(arguments, 0);
      if (isRunning) return false;
      isRunning = true;
      setTimeout(function () {
        isRunning = false;
        fn.apply(null, args);
      }, ms);
    };
  }

  function calcShift(conToImageRatio, containerSize, imageSize, focusSize, toMinus) {
    var containerCenter = Math.floor(containerSize / 2);
    var focusFactor = (focusSize + 1) / 2;
    var scaledImage = Math.floor(imageSize / conToImageRatio);
    var focus = Math.floor(focusFactor * scaledImage);
    if (toMinus) focus = scaledImage - focus;
    var focusOffset = focus - containerCenter;
    var remainder = scaledImage - focus;
    var containerRemainder = containerSize - containerCenter;
    if (remainder < containerRemainder) focusOffset -= containerRemainder - remainder;
    if (focusOffset < 0) focusOffset = 0;

    return (focusOffset * -100 / containerSize) + '%';
  }

})(Drupal, once, document, window);
